import com.typesafe.config.{Config, ConfigFactory}

import scala.xml._
import java.nio.file.{Files, Path, Paths, StandardOpenOption}
import java.time.LocalDateTime

import scala.util.{Failure, Success, Try}
import xmlvalid._
import templates._

object Main {
  val conf: Config = ConfigFactory.load
  val debugMode: Boolean = conf.getBoolean("debug")
  val logFileName: String = conf.getString("log")

  implicit def templateFactory(source: Node): Template = {
    (source \ "trade").head.child
    .filter {
      case v: Elem => true
      case _ => false
    }(1).label match {
      case "repo" => if (debugMode) new RepoTemplate(source) with Logger else new RepoTemplate(source)
      case v => throw new Exception("Undefined trade type")
    }
  }

  def main(args: Array[String]): Unit = {
    if (args.length != 2) println("Error: Undefined INPUT/OUTPUT directory. Usage: app INPUT OUTPUT")
    else if (!Files.exists(Paths.get(args(0)))) println("Error: INPUT directory doesn`t exist")
    else if (!Files.exists(Paths.get(args(1)))) println("Error: OUTPUT directory doesn`t exist")
    else {
      val p = new PrettyPrinter(140, 2)
      val nsdXmlValid = XMLValid(getClass.getResource("/nsd/nsd-ext-merged-schema.xsd"))
      val rtsXmlValid = XMLValid(getClass.getResource("/rts/fpml-recordkeeping-merged-schema.xsd"))
      var logString = ""

      for (v <- Files.walk(Paths.get(args(0))).toArray; f = v.asInstanceOf[Path]; if !Files.isDirectory(f)) {
        val fSourceName= f.getFileName.toString
        val fOutputName = s"Converted_$fSourceName"
        val sourceXml = XML.loadFile(f.toString)
        val template = Template(sourceXml)
        val outputXml = XML.loadString(p.format(template.render))
        val now = LocalDateTime.now().toString

        val sourceXmlValid = if (template.isStraight) nsdXmlValid else rtsXmlValid
        val outputXmlValid = if (template.isStraight) rtsXmlValid else nsdXmlValid

        val msgCheckSource = Try {
          sourceXmlValid.loadString(sourceXml.mkString)
        } match {
          case Success(_) => s"'$fSourceName' successfully checked against schema"
          case Failure(ex) => s"'$fSourceName' isn`t wellformed or invalid: ${ex.getMessage}"
        }

        val msgCheckOutput = Try {
          outputXmlValid.loadString(outputXml.mkString)
        } match {
          case Success(_) => s"'$fOutputName' successfully checked against schema"
          case Failure(ex) => s"'$fOutputName' isn`t wellformed or invalid: ${ex.getMessage}"
        }

        XML.save(Paths.get(args(1), fOutputName).toString, outputXml, xmlDecl = true)

        logString += template.info(s"$now\n$msgCheckSource\n$msgCheckOutput") + "\n\n"
      }

      Files.write(Paths.get(args(1), logFileName), logString.getBytes("utf-8"), StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING)
    }
  }
}
